/*
 *	CRC32 Calculator by trapflag
 *
 *	This plugin originated as a tool to calculate and display the crc32
 *	value of the code of each function in a binary/ida database.
 *	Now it displays various information such as the size of the stackframe etc.
 *
 */
 
#include <windows.h>
#include <ida.hpp>
#include <idp.hpp>
#include <bytes.hpp>
#include <loader.hpp>
#include <kernwin.hpp>
#include <allins.hpp>
#include <frame.hpp>
#include <diskio.hpp>
#include <struct.hpp>
#include <auto.hpp>


//--------------------------------------------------------------------------
//
//      Initialize.

int init(void)
{
  return PLUGIN_OK;
}

//--------------------------------------------------------------------------
//      Terminate.

void term(void)
{
//  unhook_from_notification_point(HT_UI, sample_callback);
}



//stolen from http://remus.rutgers.edu/~rhoads/Code/crc-32b.c
//and modified

static unsigned long crc_table[256];

void gen_table(void)                // build the crc table
{
    unsigned long crc, poly;
    int	i, j;

    poly = 0xEDB88320L;
    for (i = 0; i < 256; i++)
        {
        crc = i;
        for (j = 8; j > 0; j--)
            {
            if (crc & 1)
                crc = (crc >> 1) ^ poly;
            else
                crc >>= 1;
            }
        crc_table[i] = crc;
        }
}

unsigned long get_crc( ea_t startEA, ea_t endEA )    // calculate the crc value
{
    register unsigned long crc;
    int ch;
	ea_t i;

    crc = 0xFFFFFFFF;

	ua_ana0(startEA);
	cmd.ea=startEA-cmd.size;	// 'hacked' together for ua_next_byte()

	for(i=startEA;i<endEA;i++)
	{
		ch=ua_next_byte();		// get current byte
		//msg("byte at ea(%x) is %x\n",i,ch);
        crc = (crc>>8) ^ crc_table[ (crc^ch) & 0xFF ];
	}

    return( crc^0xFFFFFFFF );
}

char *yesno(bool yesno)
{
	if(yesno) return "YES";
		else return "NO";
}

void run(int arg)
{ 
	
	int numfuncs,i;
	func_t *f;
	ea_t sea,eea;
	char funcname[512];
	ulong crc32;
	ulong framesize;
	ulong argsize;
	ushort frregs;
	int retsize;
	ushort spchanges;
	struc_t *frame;
	int regargs;
	int llabels;
	ulong lvarsize;

	gen_table();	// generate crc32 table
	
	numfuncs=get_func_qty();	// get number of functions in database

	msg("\nCRC32 function calculator v0.1\n\n");

	for (i=0;i<=numfuncs-1;i++)	// first function is 0
	{

		// get function
		f = getn_func(i);
				
		// get start and end ea of function
		sea=f->startEA;
		eea=f->endEA;
		
		// get name of function
		get_func_name(sea,funcname,sizeof(funcname));
		
		// generate crc32 of function
		crc32=get_crc(sea,eea);
		
		framesize = get_frame_size(f);
		argsize = f->argsize;
		frregs = f->frregs;
		retsize = get_frame_retsize(f);
		spchanges = f->pntqty;
		regargs = f->regargqty;
		llabels = f->llabelqty;
		lvarsize = f->frsize; // = size of locals

		// display function name, current function and max function
		msg("%x: %s (%d/%d)\n",sea,funcname,i+1,numfuncs);
		msg("   Visible        [%s]\n",yesno(is_visible_func(f)));
		msg("   32bit          [%s]\n",yesno(is_32bit_func(f)));
		msg("   framesize:     %08X (complete frame)\n",framesize);
		msg("   argsize:       %08X (%d arguments?)\n",argsize,argsize/4);
		msg("   regsize:       %08X (%d saved registers)\n",frregs,frregs/4);
		msg("   frame_retsize: %08X (ESP)\n",retsize);
		msg("   varsize:       %08X (size of locals)\n",lvarsize/*framesize-frregs-retsize-argsize*/);
		msg("   sp changes:    %d\n",spchanges);
		msg("   register args: %d\n",regargs);
		msg("   local labels:  %d\n",llabels);
		msg("   CRC32:         %08X\n",crc32);
		msg("\n");
		//frame = get_frame(f);
		//frame->memqty
		
	}
	msg("\nEnd of list.\n");
	return;
	
}
//--------------------------------------------------------------------------
char comment[] = "CRC32 function Calculator";

char help[] =
        "CRC32 function Calculator\n"
        "\n"
        "This plugin calculates the CRC32 value of each function.\n"
        "\n";

//--------------------------------------------------------------------------
// This is the preferred name of the plugin module in the menu system
// The preferred name may be overriden in plugins.cfg file

char wanted_name[] = "crc32 calc";


// This is the preferred hotkey for the plugin module
// The preferred hotkey may be overriden in plugins.cfg file
// Note: IDA won't tell you if the hotkey is not correct
//       It will just disable the hotkey.

char wanted_hotkey[] = "Alt-2";


//--------------------------------------------------------------------------
//
//      PLUGIN DESCRIPTION BLOCK
//
//--------------------------------------------------------------------------

extern "C" plugin_t PLUGIN = {
  IDP_INTERFACE_VERSION,
  0,                    // plugin flags
  init,                 // initialize

  term,                 // terminate. this pointer may be NULL.

  run,                  // invoke plugin

  comment,              // long comment about the plugin
                        // it could appear in the status line
                        // or as a hint

  help,                 // multiline help about the plugin

  wanted_name,          // the preferred short name of the plugin
  wanted_hotkey         // the preferred hotkey to run the plugin
};
